/*********************************************************************
*                 SEGGER Software GmbH                               *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2018  SEGGER Microcontroller GmbH                *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.48 - Graphical user interface for embedded applications **
All  Intellectual Property rights in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product. This file may
only be used in accordance with the following terms:

The  software has  been licensed by SEGGER Software GmbH to Nuvoton Technology Corporationat the address: No. 4, Creation Rd. III, Hsinchu Science Park, Taiwan
for the purposes  of  creating  libraries  for its
Arm Cortex-M and  Arm9 32-bit microcontrollers, commercialized and distributed by Nuvoton Technology Corporation
under  the terms and conditions  of  an  End  User
License  Agreement  supplied  with  the libraries.
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information
Licensor:                 SEGGER Software GmbH
Licensed to:              Nuvoton Technology Corporation, No. 4, Creation Rd. III, Hsinchu Science Park, 30077 Hsinchu City, Taiwan
Licensed SEGGER software: emWin
License number:           GUI-00735
License model:            emWin License Agreement, signed February 27, 2018
Licensed platform:        Cortex-M and ARM9 32-bit series microcontroller designed and manufactured by or for Nuvoton Technology Corporation
----------------------------------------------------------------------
Support and Update Agreement (SUA)
SUA period:               2018-03-26 - 2019-03-27
Contact to extend SUA:    sales@segger.com
----------------------------------------------------------------------
File        : DIALOG_FS_ChooseFile.c
Purpose     : Shows how to make use of the CHOOSEFILE dialog.
              It shows the dialog with and without skinning.
              The button bar is shown at the bottom and at the top
              of the window.
Requirements: WindowManager - (x)
              MemoryDevices - (x)
              AntiAliasing  - ( )
              VNC-Server    - ( )
              PNG-Library   - ( )
              TrueTypeFonts - ( )

              Requires eithar a MS Windows environment or emFile!
----------------------------------------------------------------------
*/

#ifndef SKIP_TEST

#include "N9H30.h"
#include "sys.h"
#include "config.h"
#include "usbh_lib.h"

#include "DIALOG.h"
#include "MESSAGEBOX.h"
#undef WIN32
#ifdef WIN32
#include <windows.h>
#else
//  #include "BSP.h"
//  #include "FS.h"
//  #include "RTOS.h"

#include "string.h"

#include "ff.h"
#endif

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#ifndef _MAX_PATH
#define _MAX_PATH 256
#else
#if (_MAX_PATH > 256)
#undef  _MAX_PATH
#define _MAX_PATH 256
#endif
#endif

#ifndef _MAX_EXT
#define _MAX_EXT  256
#endif

//
// Recommended memory to run the sample with adequate performance
//
#define RECOMMENDED_MEMORY (1024L * 900)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static struct
{
    U32 Mask;
    char c;
} _aAttrib[] =
{
#ifdef WIN32
    { FILE_ATTRIBUTE_READONLY, 'R' },
    { FILE_ATTRIBUTE_HIDDEN, 'H' },
    { FILE_ATTRIBUTE_SYSTEM, 'S' },
    { FILE_ATTRIBUTE_DIRECTORY, 'D' },
    { FILE_ATTRIBUTE_ARCHIVE, 'A' }
#else
    { AM_RDO, 'R' },
    { AM_HID, 'H' },
    { AM_SYS, 'S' },
    { AM_DIR, 'D' },
    { AM_ARC, 'A' },
#endif
};

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _GetExt
*/
#ifndef WIN32
static void _GetExt(char * pFile, char * pExt)
{
    int Len;
    int i;
    int j;

    //
    // Search beginning of extension
    //
    Len = strlen(pFile);
    for (i = Len; i > 0; i--)
    {
        if (*(pFile + i) == '.')
        {
            *(pFile + i) = '\0';     // Cut extension from filename
            break;
        }
    }
    if (i == 0)
    {
        *(pExt) = '\0';          // Set end of string
        return;
    }
    //
    // Copy extension
    //
    j = 0;
    while (*(pFile + ++i) != '\0')
    {
        *(pExt + j) = *(pFile + i);
        j++;
    }
    *(pExt + j) = '\0';          // Set end of string
}
#endif

/*********************************************************************
*
*       _GetData
*/
static int _GetData(CHOOSEFILE_INFO * pInfo)
{
#ifdef WIN32
    static HANDLE hFind;
    static int NewDir;
    static char acDrive [_MAX_DRIVE];
    static char acDir   [_MAX_DIR];
    static char acName  [_MAX_FNAME];
    static char acExt   [_MAX_EXT];
    static char acMask  [_MAX_PATH];
    static char acPath  [_MAX_PATH];
    static char acAttrib[10] = {0};
    WIN32_FIND_DATA Context;
    int i, r;
    char c;

    switch (pInfo->Cmd)
    {
    case CHOOSEFILE_FINDFIRST:
        if (hFind != 0)
        {
            FindClose(hFind);
        }
        //
        // Split path into drive and directory
        //
        _splitpath(pInfo->pRoot, acDrive, acDir, NULL, NULL);
        NewDir = 1;
    //
    // Do not 'break' here...
    //
    case CHOOSEFILE_FINDNEXT:
        if (NewDir)
        {
            _makepath(acMask, acDrive, acDir, NULL, NULL);
            strcat(acMask, pInfo->pMask);
            hFind = FindFirstFile(acMask, &Context);
            if (hFind == INVALID_HANDLE_VALUE)
            {
                FindClose(hFind);
                hFind = 0;
                return 1;
            }
        }
        else
        {
            r = FindNextFile(hFind, &Context);
            if (r == 0)
            {
                FindClose(hFind);
                hFind = 0;
                return 1;
            }
        }
        NewDir = 0;
        //
        // Generate attribute string (pInfo->pAttrib)
        //
        for (i = 0; i < GUI_COUNTOF(_aAttrib); i++)
        {
            c = (Context.dwFileAttributes & _aAttrib[i].Mask) ? _aAttrib[i].c : '-';
            acAttrib[i] = c;
        }
        //
        // Make name and extension (pInfo->pName, pInfo->pExt)
        //
        if ((Context.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
        {
            _splitpath(Context.cFileName, NULL, NULL, acName, acExt);
        }
        else
        {
            strcpy(acName, Context.cFileName);
            acExt[0] = 0;
        }
        //
        // Pass data to dialog
        //
        pInfo->pAttrib = acAttrib;
        pInfo->pName   = acName;
        pInfo->pExt    = acExt;
        pInfo->SizeL   = Context.nFileSizeLow;
        pInfo->SizeH   = Context.nFileSizeHigh;
        pInfo->Flags   = (Context.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
                         ? CHOOSEFILE_FLAG_DIRECTORY : 0;
    }
    return 0;
#else
#if 0
    static FS_FIND_DATA FindData;
    static char         acAttrib[10]         = { 0 };
    static char         acFile  [_MAX_PATH];
    static char         acExt   [_MAX_EXT];
    char                c;
    int                 i;
    int                 r;

    switch (pInfo->Cmd)
    {
    case CHOOSEFILE_FINDFIRST:
        r = FS_FindFirstFile(&FindData, pInfo->pRoot, acFile, sizeof(acFile));
        break;
    case CHOOSEFILE_FINDNEXT:
        r = FS_FindNextFile(&FindData) ^ 1;
        break;
    }
    if (r == 0)
    {
        pInfo->Flags = (FindData.Attributes & _aAttrib[3].Mask) ? CHOOSEFILE_FLAG_DIRECTORY : 0;
        for (i = 0; i < GUI_COUNTOF(_aAttrib); i++)
        {
            if (FindData.Attributes & _aAttrib[i].Mask)
            {
                c = _aAttrib[i].c;
            }
            else
            {
                c = '-';
            }
            acAttrib[i] = c;
        }
        _GetExt(acFile, acExt);
        pInfo->pAttrib = acAttrib;
        pInfo->pName   = acFile;
        pInfo->pExt    = acExt;
        pInfo->SizeL   = FindData.FileSize;
        pInfo->SizeH   = 0;
    }
    return r;
#else
    static char         acAttrib[10]         = { 0 };
    static char         acFile  [_MAX_PATH];
    static char         acExt   [_MAX_EXT];
    static char         acPath  [_MAX_PATH];
    char                c;
    int                 i;
    int                 r;
    FRESULT fr;     /* Return value */
    static DIR dj;         /* Directory search object */
    FILINFO fno;    /* File information */

    switch (pInfo->Cmd)
    {
    case CHOOSEFILE_FINDFIRST:
        strcpy(acPath, pInfo->pRoot);
        r = strlen(acPath);
        acPath[--r] = 0x00;
        fr = f_findfirst(&dj, &fno, acPath, "*");  /* Start to search for photo files */
        if (fr == FR_OK && fno.fname[0])           /* Repeat while an item is found */
        {
//            sysprintf("%s\n", fno.fname);                /* Display the object name */
            strcpy(acFile, fno.fname);
            pInfo->Flags = (fno.fattrib & _aAttrib[3].Mask) ? CHOOSEFILE_FLAG_DIRECTORY : 0;
            for (i = 0; i < GUI_COUNTOF(_aAttrib); i++)
            {
                if (fno.fattrib & _aAttrib[i].Mask)
                {
                    c = _aAttrib[i].c;
                }
                else
                {
                    c = '-';
                }
                acAttrib[i] = c;
            }
            _GetExt(acFile, acExt);
            pInfo->pAttrib = acAttrib;
            pInfo->pName   = acFile;
            pInfo->pExt    = acExt;
            pInfo->SizeL   = fno.fsize;
            pInfo->SizeH   = 0;
            return 0;
        }
        break;
    case CHOOSEFILE_FINDNEXT:
        fr = f_findnext(&dj, &fno);               /* Search for next item */
        if (fr == FR_OK && fno.fname[0])           /* Repeat while an item is found */
        {
//            sysprintf("%s\n", fno.fname);                /* Display the object name */
            strcpy(acFile, fno.fname);
            pInfo->Flags = (fno.fattrib & _aAttrib[3].Mask) ? CHOOSEFILE_FLAG_DIRECTORY : 0;
            for (i = 0; i < GUI_COUNTOF(_aAttrib); i++)
            {
                if (fno.fattrib & _aAttrib[i].Mask)
                {
                    c = _aAttrib[i].c;
                }
                else
                {
                    c = '-';
                }
                acAttrib[i] = c;
            }
            _GetExt(acFile, acExt);
            pInfo->pAttrib = acAttrib;
            pInfo->pName   = acFile;
            pInfo->pExt    = acExt;
            pInfo->SizeL   = fno.fsize;
            pInfo->SizeH   = 0;
            return 0;
        }
        else
        {
            f_closedir(&dj);
//            return 1;
        }
//        break;
    }
#endif
#endif
    return 1;
}

/*********************************************************************
*
*       _EnableSkinning
*/
static void _EnableSkinning(int OnOff)
{
    if (OnOff)
    {
        FRAMEWIN_SetDefaultSkin(FRAMEWIN_SKIN_FLEX);
        BUTTON_SetDefaultSkin  (BUTTON_SKIN_FLEX);
        DROPDOWN_SetDefaultSkin(DROPDOWN_SKIN_FLEX);
        HEADER_SetDefaultSkin(HEADER_SKIN_FLEX);
        SCROLLBAR_SetDefaultSkin(SCROLLBAR_SKIN_FLEX);
    }
    else
    {
        FRAMEWIN_SetDefaultSkinClassic();
        BUTTON_SetDefaultSkinClassic();
        DROPDOWN_SetDefaultSkinClassic();
        HEADER_SetDefaultSkinClassic();
        SCROLLBAR_SetDefaultSkinClassic();
    }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

static char _acBuffer[4 * 1024] __attribute__((aligned(32)));

extern volatile int end_flag;

/*********************************************************************
*
*       _GetDataBMP
*
* Function description
*   This routine is called by the functions GUI_BMP_GetXSizeEx(),
*   GUI_BMP_GetYSizeEx() and GUI_BMP_DrawEx(). The routine is responsible
*   for setting the data pointer to a valid data location with at least
*   one valid byte.
*
* Parameters
*   p           - Pointer to application defined data.
*   NumBytesReq - Number of bytes requested.
*   ppData      - Pointer to data pointer. This pointer should be set to
*                 a valid location.
*   StartOfFile - If this flag is 1, the data pointer should be set to the
*                 beginning of the data stream.
*
* Return value
*   Number of data bytes available.
*/
static int _GetDataBMP(void * p, const U8 ** ppData, unsigned NumBytesReq, U32 Off)
{
    U32 NumBytesRead;
    FRESULT res;
    FIL hFile1;

    hFile1 = *(FIL *)p;
    //
    // Check buffer size
    //
    if (NumBytesReq > sizeof(_acBuffer))
    {
        NumBytesReq = sizeof(_acBuffer);
    }
    //
    // Set file pointer to the required position
    //
    f_lseek(&hFile1, Off);
    //
    // Read data into buffer
    //
    res = f_read(&hFile1, _acBuffer, NumBytesReq, &NumBytesRead);
    //
    // Set data pointer to the beginning of the buffer
    //
    *ppData = _acBuffer;
    //
    // Return number of available bytes
    //
    return NumBytesRead;
}

static void _ShowBMP(const char * sFilename)
{
    FIL              hFile;
#define PARAM    &hFile
//  int    XSize;
//  int    YSize;
//  int    XPos;
//  int    YPos;

    f_open(PARAM, sFilename, FA_OPEN_EXISTING | FA_READ);
//  GUI_ClearRect(0, 60, 319, 239);
//  XSize = GUI_BMP_GetXSizeEx(_GetDataBMP, PARAM);
//  YSize = GUI_BMP_GetYSizeEx(_GetDataBMP, PARAM);
//  XPos  = (XSize > 320) ?  0 : 160 - (XSize / 2);
//  YPos  = (YSize > 180) ? 60 : 150 - (YSize / 2);
//  if (!GUI_BMP_DrawEx(_GetDataBMP, PARAM, XPos, YPos)) {
//    end_flag = 0;
    GUI_MULTIBUF_Begin();
    GUI_BMP_DrawEx(_GetDataBMP, PARAM, 0, 0);
    GUI_MULTIBUF_End();
//        while ( end_flag == 0 );
    f_close(PARAM);
}

static void _ShowJPG(const char * sFilename)
{
    FIL              hFile;
#define PARAM    &hFile
//  int    XSize;
//  int    YSize;
//  int    XPos;
//  int    YPos;

    f_open(PARAM, sFilename, FA_OPEN_EXISTING | FA_READ);
//  GUI_ClearRect(0, 60, 319, 239);
//  XSize = GUI_BMP_GetXSizeEx(_GetDataBMP, PARAM);
//  YSize = GUI_BMP_GetYSizeEx(_GetDataBMP, PARAM);
//  XPos  = (XSize > 320) ?  0 : 160 - (XSize / 2);
//  YPos  = (YSize > 180) ? 60 : 150 - (YSize / 2);
//  if (!GUI_BMP_DrawEx(_GetDataBMP, PARAM, XPos, YPos)) {
//    end_flag = 0;
    GUI_MULTIBUF_Begin();
    GUI_JPEG_DrawEx(_GetDataBMP, PARAM, 0, 0);
    GUI_MULTIBUF_End();
//        while ( end_flag == 0 );
    f_close(PARAM);
}

static char *getExtension(char *fileName)
{
    int len;
    int i;

    len = strlen(fileName);
    i = len;
    while ( fileName[i] != '.' && i > 0)
        i--;

    if ( fileName[i] == '.' )
        return &fileName[i+1];
    else
        return &fileName[len];
}
#if 0
WCHAR ff_convert (WCHAR wch, UINT dir)
{
    if (wch < 0x80)
    {
        /* ASCII Char */
        return wch;
    }

    /* I don't support unicode it is too big! */
    return 0;
}

WCHAR ff_wtoupper (WCHAR wch)
{
    if (wch < 0x80)
    {
        /* ASCII Char */
        if (wch >= 'a' && wch <= 'z')
        {
            wch &= ~0x20;
        }
        return wch;
    }

    /* I don't support unicode it is too big! */
    return 0;
}
#endif

#include "WM.h"

/*******************************************************************
*
*       _cbWindow1
*/
CHOOSEFILE_INFO   Info                = { 0 };
static void _cbWindow1(WM_MESSAGE * pMsg)
{
    switch (pMsg->MsgId)
    {
    case WM_PAINT:
        sysprintf("%s %s\n", Info.pRoot, getExtension(Info.pRoot));
        getExtension(Info.pRoot);
        if (strcasecmp(getExtension(Info.pRoot), "bmp") == 0)
            _ShowBMP(Info.pRoot);
        else
            sysprintf("not bmp\n");
        if (strcasecmp(getExtension(Info.pRoot), "jpg") == 0)
            _ShowJPG(Info.pRoot);
        else
            sysprintf("not jpg\n");
        break;
    default:
        WM_DefaultProc(pMsg);
    }
}

/*******************************************************************
*
*       _DemoCreateWindow
*
* Function description
*   Demonstrates the use of WM_CreateWindow
*/

//
// Handles
//
static WM_HWIN _hWindow1;

static void _DemoCreateWindow(void)
{
    //
    // Create windows
    //
    _hWindow1 = WM_CreateWindow(0, 0, 800, 480, WM_CF_SHOW, _cbWindow1, 0);
}

/*********************************************************************
*
*       MainTask
*/
void MainTask(void)
{
#if 1
    const char        acMask[]            = "*.*";
//  CHOOSEFILE_INFO   Info                = { 0 };
    WM_HWIN           hWin;
    int               xSize;
    int               ySize;
    int               Cnt                 = 0;
    int               r;
#ifdef WIN32
    char              acDirLocal[]        = "C:\\";
    char              acDirWin[_MAX_PATH] = "\0";
    char              acDirTmp[_MAX_PATH] = "\0";
    char              acDirSys[_MAX_PATH] = "\0";
    char            * apDrives[4]         = { acDirLocal, acDirWin, acDirTmp, acDirSys };
#else
    char              acDir[_MAX_PATH]    = "3:\\";
    char const      * apDrives[1]         = { acDir };
#endif
    FRESULT res;
    DIR dir;
//    usbh_pooling_hubs();

#ifndef WIN32
    //
    // Initialize emFile
    //
//  FS_Init();
//  FS_FAT_SupportLFN();
#else
    //
    // Initialize directories
    //
    GetWindowsDirectory(acDirWin, sizeof(acDirWin));
    GetTempPath        (sizeof(acDirTmp), acDirTmp);
    GetSystemDirectory (acDirSys, sizeof(acDirSys));
#endif
    //
    // Initialize emWin
    //
//  WM_SetCreateFlags(WM_CF_MEMDEV);
//  GUI_Init();
//  WM_MULTIBUF_Enable(1);
    //
    // Check if recommended memory for the sample is available
    //
    if (GUI_ALLOC_GetNumFreeBytes() < RECOMMENDED_MEMORY)
    {
        GUI_ErrorOut("Not enough memory available.");
        return;
    }
    WM_SetDesktopColor(GUI_WHITE);
    //
    // Set defaults for FRAMEWIN widget
    //
    FRAMEWIN_SetDefaultFont(GUI_FONT_20_ASCII);
    FRAMEWIN_SetDefaultTextAlign(GUI_TA_HCENTER);
    //
    // Get display size and initialize pointers to search mask and callback routine
    //
    xSize = 320;
    ySize = 240;
    Info.pfGetData = _GetData;
    Info.pMask     = acMask;
    while (1)
    {
        res = f_opendir(&dir, "3:\\");
        if (res)
        {
            sysprintf("### no USB pen drive!\n");
            GUI_SetBkColor(GUI_WHITE);
            GUI_Clear();
            GUI_MessageBox("Please plug-in USB pen drive!",
                       "Warning", GUI_MESSAGEBOX_CF_MOVEABLE);
            GUI_SetBkColor(GUI_WHITE);
            GUI_Clear();
            GUI_SetColor(GUI_GREEN);
            GUI_SetFont(GUI_FONT_32B_ASCII);
            GUI_DispStringHCenterAt("Reading... please wait!", 400, 240);
            usbh_pooling_hubs();
            GUI_SetBkColor(GUI_WHITE);
            GUI_Clear();
            continue;
        }
        else
        {
            GUI_SetBkColor(GUI_WHITE);
            GUI_Clear();
            GUI_SetColor(GUI_BLUE);
            GUI_SetFont(GUI_FONT_32B_ASCII);
            GUI_DispStringHCenterAt("Reading... please wait!", 400, 240);
            f_closedir(&dir);
            usbh_pooling_hubs();
            GUI_SetBkColor(GUI_WHITE);
            GUI_Clear();
        }
        sysprintf("EHCI: 0x%x, 0x%x;  OHCI: 0x%x, 0x%x\n", HSUSBH->UPSCR[0], HSUSBH->UPSCR[1], USBH->HcRhPortStatus[0], USBH->HcRhPortStatus[1]);
//        usbh_pooling_hubs();
//      while (sysIsKbHit() == 0)
//            usbh_pooling_hubs();

        _EnableSkinning(((Cnt >> 2) % 2) ^ 1);
        CHOOSEFILE_SetTopMode(Cnt % 2);
        hWin = CHOOSEFILE_Create(WM_HBKWIN, 0, 0, xSize, ySize, apDrives, GUI_COUNTOF(apDrives), 0, "File Dialog", 0, &Info);
        //
        // Show different variants of dialog
        //
        if (((Cnt >> 1) % 2) == 0)
        {
            CHOOSEFILE_SetButtonText(hWin, CHOOSEFILE_BI_OK, "OK");
            CHOOSEFILE_SetButtonText(hWin, CHOOSEFILE_BI_CANCEL, "Cancel");
            if (((Cnt >> 2) % 2) == 0)
            {
                CHOOSEFILE_SetButtonText(hWin, CHOOSEFILE_BI_UP, "UP");
            }
        }
        r = GUI_ExecCreatedDialog(hWin);
        _DemoCreateWindow();
        GUI_Delay(3000);
        WM_DeleteWindow(_hWindow1);
#if 0
        if (r == 0)
        {
//      hWin = MESSAGEBOX_Create(Info.pRoot, "File:", 0);
            sysprintf("%s %s\n", Info.pRoot, getExtension(Info.pRoot));
            getExtension(Info.pRoot);
            if (strcasecmp(getExtension(Info.pRoot), "bmp") == 0)
                _ShowBMP(Info.pRoot);
            else
                sysprintf("not bmp\n");
            if (strcasecmp(getExtension(Info.pRoot), "jpg") == 0)
                _ShowJPG(Info.pRoot);
            else
                sysprintf("not jpg\n");
//      GUI_Delay(2000);
//      WM_DeleteWindow(hWin);
        }
        else
        {
            GUI_Delay(500);
        }
#endif
        Cnt++;
    }
#else
    while (1)
    {
        sysprintf("EHCI: 0x%x, 0x%x;  OHCI: 0x%x, 0x%x\n", HSUSBH->UPSCR[0], HSUSBH->UPSCR[1], USBH->HcRhPortStatus[0], USBH->HcRhPortStatus[1]);
        usbh_pooling_hubs();
        _ShowJPG("3:\\p0.jpg");
        GUI_Delay(2000);
        sysprintf("EHCI: 0x%x, 0x%x;  OHCI: 0x%x, 0x%x\n", HSUSBH->UPSCR[0], HSUSBH->UPSCR[1], USBH->HcRhPortStatus[0], USBH->HcRhPortStatus[1]);
        usbh_pooling_hubs();
        _ShowJPG("3:\\p1.jpg");
        GUI_Delay(2000);
        sysprintf("EHCI: 0x%x, 0x%x;  OHCI: 0x%x, 0x%x\n", HSUSBH->UPSCR[0], HSUSBH->UPSCR[1], USBH->HcRhPortStatus[0], USBH->HcRhPortStatus[1]);
        usbh_pooling_hubs();
        _ShowJPG("3:\\p2.jpg");
        GUI_Delay(2000);
        sysprintf("EHCI: 0x%x, 0x%x;  OHCI: 0x%x, 0x%x\n", HSUSBH->UPSCR[0], HSUSBH->UPSCR[1], USBH->HcRhPortStatus[0], USBH->HcRhPortStatus[1]);
        usbh_pooling_hubs();
        _ShowJPG("3:\\p3.jpg");
        GUI_Delay(2000);
    }
#endif
}

#endif

/*************************** End of file ****************************/
